;Simple Window Mask, Color Add/Sub, and HDMA Demo
;(C) 2001 Realtime Simulations and Roleplaying games, Inc.
;Coding by Grog

	;Program code starts at $00:8000 (first ROM address)
        .ORG $8000
        sei                     ;disable interrupts
        clc                     ;switch to native mode
        xce

        JSR InitializeSNES

        rep #$30                ;8b mem, 16b X
        sep #$20
        .mem 8
        .index 16

        ;Load Tiles and palette from Tiledata
        .LoadBlockToVRAM ($00,Tiledata,$1000,$6400) ;25600 bytes
        .LoadNewPalette ($00,Tiledata+25600, 0, 16) ;Colors at end of file

	JSR DrawMap
   JSR SetupVideo

   ;Setup the color window
   JSR SetupColorWindow

   lda #$80
   sta $4200   ;Enable VBlank NMI

InfiniteLoop:
	BRA InfiniteLoop	;Wait forever, displaying the image

VBlank:

   ;Reset the HDMA
   STZ $420C         ;Turn off HDMA
   JSR SetupHDMA     ;Setup the HDMA transfers on Channels 6 and 7
   LDA #$C0 
   STA $420C         ;Activate HDMA channels 6 and 7

   lda $4210         ;Clear NMI flag
   RTI               ;Done!


	;ROM header definitions
        .interrupts
            all = empty_irq
            reset = restart
            nmi = VBlank
        .end

        empty_irq       rti
        restart         jmp $008000

        .cartridge
            title           =       "Simple HDMA"
            mode            =       $20				;LoROM mode
            type            =       $01                         ;no special chips or SRAM
            romsize         =       $08                         ;32k ROM (256kbit)
            ramsize         =       $00                         ;0k SRAM
            country         =       $01				;USA
            maker           =       $00				;Not set (use $13 for RSR)
            version         =       $05            ;Revision of ROM
        .end
;End of demo Main code

;============================================================================
; DrawMap -- Draw a simple tile map that will display the image
;----------------------------------------------------------------------------
; In: None
;----------------------------------------------------------------------------
; Out: None
;----------------------------------------------------------------------------
; Modifies: none
;----------------------------------------------------------------------------
DrawMap:	;Assumes mem8, index16
	phx
        stz $2116               ;reset VRAM pointer to $0000
        stz $2117
        ldx #$0000              ;init counter
-       stx $2118               ;store the tile into VRAM
        inx                     ;next tile
        cpx #$0800              ;loop until 2048 tiles have been drawn
        bne -
	plx
	rts
;============================================================================
; SetupVideo -- Set the video mode for the demo
;----------------------------------------------------------------------------
; In: None
;----------------------------------------------------------------------------
; Out: None
;----------------------------------------------------------------------------
; Modifies: none
;----------------------------------------------------------------------------
SetupVideo:	;Assumes mem8, index16
	php
	pha
   lda #$01
   sta $2105               ;Set video mode 1 (16 color BG1)
   lda #$11
	sta $210B		;Set BG1's Tile VRAM offset to $1000
   lda #$01
   sta $212C    ;Turn on BG1
   lda #$0F
   sta $2100    ;Turn on screen, full brightness
	pla
	plp
	rts
;============================================================================

;============================================================================
; SetupHDMA -- Setup the HDMA for the fixed color subtract window
;----------------------------------------------------------------------------
; In: None
;----------------------------------------------------------------------------
; Out: None
;----------------------------------------------------------------------------
; Modifies: none
;----------------------------------------------------------------------------
SetupHDMA:
   LDX #HDMATableLeft
   STX $4372
   LDX #HDMATableRight
   STX $4362
   LDA #$00
   STA $4374
   STA $4364
   LDA #$26
   STA $4371
   LDA #$27
   STA $4361
   LDA #$00
   STA $4370
   STA $4360
   RTS

;This HDMA table controls the Left side of the color window
HDMATableLeft:
   .DB $28,$FF    ;Set left=255, skip to line 40
   .DB $64,$20    ;Set left=32, skip to line 140
   .DB $01,$FF    ;Set left=255
   .DB 0

;This HDMA table controls the Right side of the color window
HDMATableRight:
   .DB $28,$00    ;Set right=0, skip to line 40
   .DB $64,$D0    ;Set right=208, skip to line 140
   .DB $01,$FF    ;Set right=0
   .DB 0
;============================================================================

;============================================================================
; SetupColorWindow -- setup the fixed color subtraction window
;----------------------------------------------------------------------------
; In: None
;----------------------------------------------------------------------------
; Out: None
;----------------------------------------------------------------------------
; Modifies: none
;----------------------------------------------------------------------------
SetupColorWindow:
   lda #$A0
   sta $2125      ;Enable Color Window 1
   lda #$FF
   sta $2126      ;Set Window 1 left position
   lda #$D0
   sta $2127      ;Set Window 1 right position
   lda #$10
   sta $2130      ;Color addition selection
   lda #$81
   sta $2131      ;Add/Sub
   lda #$EC
   sta $2132      ;Enable R, G, B at dim color constant
   RTS
;============================================================================



;Include Library functions
	.INCSRC "INITSNES.ASM"
	.INCSRC "LOADPAL.ASM"
	.INCSRC "LOADVRAM.ASM"

;Binary data (Tiles, colors)
Tiledata
        .incbin "DEMOTILE.BIN"

